/*
 * Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This file is part of Qt Web Runtime.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

if ((typeof wrt.device.sysinfo) == 'undefined') {
    /**
     * A wrapper JS class for objects implementing ISysInfo interface of SysInfo Plugin
     *
     * @namespace
     */
    wrt.device.sysinfo = (function(){

            var DeviceException = wrt.device.DeviceException;
            var error = new DeviceException(0, 'dummy');
            /*
             * Internal
             * qt sysinfo interface object.
             */
            var transactionIdstChannel = null; // to be stored when startchannel is called, use this while stop channel is called to remove the signal
            var symbianPlatform = "Symbian";
            var s60Platform = "Series60";
            var qtSysInfoIf = wrt._services.load("wrt.device.sysinfo", "com.nokia.ISysInfo", "1.0");	

            qtSysInfoIf.addEventListener("asyncCallback(int,int,const QMap<QString,QVariant>&)", sysInfo_getChannel_cb);
            qtSysInfoIf.addEventListener("asyncCallbackStChannel(int,int,const QMap<QString,QVariant>&)", sysInfo_startChannel_cb);


            /*
             * SysInfo getChannel callback handler
             */
            function sysInfo_getChannel_cb(errCode, transId, result){

                var retObj = asyncCallbackMap.get(transId);
                asyncCallbackMap.remove(transId);

                if (!retObj) {
                    return;
                }
                var callback = retObj.success_cb;
                var errorCallback = retObj.error_cb;
                if (errCode != 0) {
                    errorCallback(new DeviceException( errCode, "Error in async operation" ) );
                    return;
                }
                callback(result);
            }

            /*
             * SysInfo startChannel callback handler
             */
            function sysInfo_startChannel_cb(errCode, transId, result){

                var retObj = asyncCallbackMap.get(transId);
                if (!retObj) {
                    return;
                }
                var callback = retObj.success_cb;
                var errorCallback = retObj.error_cb;
                if (errCode != 0) {
                    errorCallback( new DeviceException( errCode , "Error in async operation" ) );
                    return;
                }
                callback(result);
            }

            /*
             * Internal
             * sysInfoObject object which implements ISysInfo interface of SysInfoService
             */
            function SysInfo(qtSysInfoIf){
                this._sysInfoObject = qtSysInfoIf;
            }

            return {
                /**
                 * @constant {String}
                 * interfaceName.
                 */
interfaceName: "sysinfo",
                   /**
                    * Gets the list of system information channels supported on the device.
                    * This is a synchronous function.
                    * @return {array} Channel descriptors(as maps) for the available channels }
                    */
getChannelList : function(){
		if(arguments.length != 0 )
		{
		 throw new  DeviceException(error.INVALID_ARG_ERR,"An argument passed is not an expected type and is invalid");
		}
                 return qtSysInfoIf.getChannelList();
                },
                /**
                 * Sets the properties of the specified channel.
                 * This is a synchronous function.
                 * @param {string} aChannel - Name of the channel as specified in the channel
                 *                            descriptor.The channel is DisplayWallpaper
                 * @param {list} inputData - input data list of the set.
                 * @return {object} error object
                 * @throws {DeviceException} Code 1 (MISSING_ARG_ERR): channel is undefined.<br>
                 *                           Code 2 (INVALID_ARG_ERR): channel is not a string.<br>
                 */
setChannel : function(aChannel, inputData) {
		var retVal;
		if(aChannel == null ||  aChannel == 'undefined')
		  	throw new DeviceException(error.MISSING_ARG_ERR,"A mandatory argument in a method is missing");	
		   else if(typeof aChannel != 'string')
		  	throw new DeviceException(error.INVALID_ARG_ERR,"An argument passed is not an expected type and is invalid");	
		   else
                      retVal = qtSysInfoIf.setChannelSync( aChannel, inputData );
				      	
                 if ( retVal.errCode != 0 ){
                     throw new DeviceException( retVal.errCode, retVal.errMessage );                                                                   
                 }                           
               return retVal;
             },
             /**
              * Gets the properties of the specified channel.
              * This method supports synchronous and asynchronous based on style information of <i>channel descriptor</i>.
              * This is a synchronous/asynchronous function.
              * @param {string} aChannel - Name of the channel as specified in the <i>channel
              *                        descriptor</i>.The synchronous channels are Charging,
              *                        PhoneLanguage,BluetoothOn,ProductType and FirmwareVersion.
              *                        The asynchronous channels are BatteryLevel,Network
              *                            and SignalStrength.
              * @param {function} [successCB] - callback function invoked if async call
              *                       is completed and successful.<br><ul>function signature <b>function (Sysinfo data)</b></ul>
              * @param {function} [errorCB(DeviceError)] - callback function invoked if async call is
              *                     completed and not successful.
              *
              * <br><br>
              * <b>Status Codes</b> are the codes returned in errorCallback functionErrorCallback for channels that support asynchronous behavior.<br>
              * <br>
              *                          Code 101 (DATA_NOT_FOUND_ERR): If there is no active asynchronous request corresponding to the specified transactionId.
              *                          <br>
              *
              * @return {Sysinfo Data/number} Properties of the channel(if synchronous)
              *               or transaction id(if asynchronous)
              * @throws {DeviceException} Code 1 (MISSING_ARG_ERR): channel is undefined.<br>
              *                           Code 2 (INVALID_ARG_ERR): channel is not a string.<br>
              *                           Code 101 (DATA_NOT_FOUND_ERR): The specified channel is not available.
              */
getChannel : function(aChannel, successCB, errorCB) 
             {
                 var retVal;
                 if ( successCB == null )                                                                                                     
                 {                              
	             if(aChannel == null ||  aChannel == 'undefined')
		  	       throw new DeviceException(error.MISSING_ARG_ERR,"A mandatory argument in a method is missing");	
		         else if(typeof aChannel != 'string')
		  	       throw new DeviceException(error.INVALID_ARG_ERR,"An argument passed is not an expected type and is invalid");	
		         else
                   retVal = qtSysInfoIf.getChannelSync(aChannel);   
		                                                                           
			 if(retVal.errCode =='undefined')
			throw new DeviceException(  error.DATA_NOT_FOUND_ERR ,"The specified channel is not available");                                                           
                     if ( retVal.errCode != 0 )     	                                                                                    
                     {                              
					 	throw new DeviceException( retVal.errCode, retVal.errMessage );                                        				                                                                                      
                         
                     }                                                                                                                              
                 }                                                                                                                            
                 else                                                                                             
                 {                                                                                                                                      
                     //If callback is not a function then throw an exception                                        
                     if ( typeof successCB != "function" || successCB == undefined )                                
                     {                                                                                              
                         throw new DeviceException( error.INVALID_ARG_ERR, "Success callback parameter is not a function" );              
                     }                                                                                                                    
                     if ( errorCB && ( typeof errorCB != "function" ) )                                                                             
                     {                                                                                                                              
                         throw new DeviceException( error.INVALID_ARG_ERR, "Error callback parameter is not a function" );                          
                     }                                                                                                                              

		    if(aChannel == null ||  aChannel == 'undefined')
		  	throw new DeviceException(error.MISSING_ARG_ERR,"A mandatory argument in a method is missing");	
		    else if(typeof aChannel != 'string')
		  	throw new DeviceException(error.INVALID_ARG_ERR,"An argument passed is not an expected type and is invalid");	
		    else {
			 var tid = asyncCallbackMap.add(successCB, errorCB);     
    	             	 retVal = qtSysInfoIf.getChannelAsync(aChannel, tid);    
		    } 
                                                                        
                     if ( retVal.errCode != 0 )                                                                             
                     {                                                                                                                              
                         asyncCallbackMap.remove(tid);                                                                                    
                         throw new DeviceException( retVal.errCode, retVal.errMessage );                                                          
                     }                                                                                                                              
	             else
			 return retVal.transactionId; 
                 }                                                                                                                            
                 if (aChannel == "ListDriveInfo")                                                                                                       
                 {                                                                                                                                      
                     return retVal.listOfDrives;                                                                                                    
                 }                                                                                                                            

                 return retVal; 

             },

             /**
              * Registers for notifications from a particular channel.
              * This is an asynchronous method. If the notification for a particular
              * channel has already been registered, and subsequently a call made again,
              * then the new handler will replace the older call(s).
              * @param {function} successCB - callback function called on successful async completion
              * @param {string} aChannel - an string value
              * @param {int/object} aTrigger(optional) - trigger param
              * @param {function} errorCB - callback function called on un-successful async completion
              * @return {void}
              * @throws {DeviceException} Code 1 (MISSING_ARG_ERR): channel is undefined.<br>
              *                           Code 2 (INVALID_ARG_ERR): channel is not a string.<br>
              */
startChannel : function(successCB, aChannel, aTrigger, errorCB) 
               {
               		 var retVal;
                   if ( successCB == null )
                   {                                                                                                                      
                       throw new DeviceException( error.MISSING_ARG_ERR, "Success callback parameter missing" );                                                      
                   }                                                                                                                                          
                   if ( typeof successCB != "function" )
                   {                                                                                   
                       throw new DeviceException( error.INVALID_ARG_ERR, "Success callback parameter is not a function" );                                            
                   }                                                                                                                  


                   if(aChannel == null ||  aChannel == 'undefined')
		  	throw new DeviceException(error.MISSING_ARG_ERR,"A mandatory argument in a method is missing");	
		   else if(typeof aChannel != 'string')
		  	throw new DeviceException(error.INVALID_ARG_ERR,"An argument passed is not an expected type and is invalid");	
		   else {			   
			var tid = asyncCallbackMap.add(successCB, errorCB);
	                retVal = qtSysInfoIf.startChannel(aChannel,tid,aTrigger); 
                   }                                                                             
		   
                   if ( retVal.errCode != 0)                                                                                                        
                   {                                                                                                                            
                       asyncCallbackMap.remove(tid);                                                                                                      
                       throw new DeviceException( retVal.errCode, retVal.errMessage);                                                                     
                   }    

               },

               /**
                * Stops the notifications (started by startChannel) for the channel specified.
                * This is a synchronous method.
                * @function {public var} stopChannel
                * @param {string} aChannel - an string value
                * @return {void}
                * @throws {DeviceException} Code 1 (MISSING_ARG_ERR): channel is undefined.<br>
                *                           Code 2 (INVALID_ARG_ERR): channel is not a string.<br>
                */
stopChannel : function(aChannel)
              { 
              	var retVal;
	          if(aChannel == null ||  aChannel == 'undefined')
		  	throw new DeviceException(error.MISSING_ARG_ERR,"A mandatory argument in a method is missing");	
		   else if(typeof aChannel != 'string')
		  	throw new DeviceException(error.INVALID_ARG_ERR,"An argument passed is not an expected type and is invalid");	
		   else
	                retVal = qtSysInfoIf.stopChannel(aChannel);			                                                                                        
                                                     
                  //If provider call fails, then throw error                                                                                             
                  if ( retVal.errCode != 0 )
                  {
                      asyncCallbackMap.remove(transactionIdstChannel);   
                      throw new DeviceException( retVal.errCode, retVal.errMessage );                                                                    
                  }  
              },

              /**
               * Cancels async requests made in getChannel
               * @param {int} aTransactionId - integer transaction id
               * @return {void}
               * @throws {DeviceException} Code 1 (MISSING_ARG_ERR): id is undefined.<br>
               *                           Code 2 (INVALID_ARG_ERR): id is not a number.<br>
               */
cancel : function(aTransactionId)
         {
             if(aTransactionId == null || aTransactionId == 'undefined')
		throw new DeviceException(error.MISSING_ARG_ERR,"A mandatory argument in a method is missing")
	      
             var cancelObj = asyncCallbackMap.get(aTransactionId);                                                                                          
             if (cancelObj != undefined)
             {                                                                                                                                 
                 var retVal = qtSysInfoIf.cancel(aTransactionId);                                                                                              
                 if ( retVal.errCode != 0 )
                 {
                     throw new DeviceException( retVal.errCode, retVal.errMessage );                                                                    
                 }
                 else
                     asyncCallbackMap.remove(aTransactionId);                                                                                                     
             }                                                                                                                                              
             else 
             {                                                                                                                               
                 throw new DeviceException( error.INVALID_ARG_ERR, "Transaction id not found");                                                               
             } 
	     
         }                                                                                
         }; //End return
})();//End sensors
}// end if
